function crossRatio()
clear all
close all

set(0,'defaultTextFontName', 'Arial');
set(0,'defaultTextFontSize', 14);
set(0,'defaultTextFontWeight', 'Bold');
set(0,'defaultTextBackgroundColor', [1,1,1]);
set(0,'defaultLineLineWidth', 3);

im = imread('subbuteoField-crop.jpg');
imOrig = imread('subbuteoField.jpg');


bpos = [164.0714  373.2019 1]';
bneg = [86.8665  183.9472 1]';

apos = [143.5745  322.6878 1]';
aneg = [102.8273  223.0723 1]';

d = bpos - bneg;
d = d/norm(d);

theta_bpos = norm(bpos - bneg);

theta_apos = norm(apos - bneg);

theta_aneg = norm(aneg - bneg);

[theta_center, theta_vanishing] = doubleCross(theta_apos, theta_aneg, theta_bpos, 0);

point_center = hnormalise(bneg + theta_center * d);
point_vanishing = hnormalise(bneg + theta_vanishing * d);

figure(), imshow(im);
hold on
plot(bpos(1), bpos(2), 'r+');text(bpos(1)-10, bpos(2), 'b_+','HorizontalAlignment','right','VerticalAlignment','middle');
plot(bneg(1), bneg(2), 'r+');text(bneg(1)-10, bneg(2), 'b_-','HorizontalAlignment','right','VerticalAlignment','middle');

plot(apos(1), apos(2), 'b+');text(apos(1)-10, apos(2), 'a_+','HorizontalAlignment','right','VerticalAlignment','middle');
plot(aneg(1), aneg(2), 'b+');text(aneg(1)-10, aneg(2), 'a_-','HorizontalAlignment','right','VerticalAlignment','middle');

% plot(point_center(1), point_center(2), 'g+');
% plot(point_vanishing(1), point_vanishing(2), 'g+');
axis auto
hold off

% to check, plot on the original image and use a point on the field line 
% graphically check that vanishing is true
pchk = [684.6745  208.6276 1]';

% and check that the center is in the middle usign coordinate of the
% penalty point on the other side
ppen = [581.6171  151.4630 1]';

figure(), imshow(imOrig);
hold on

plot([bpos(1) point_vanishing(1)], [bpos(2) point_vanishing(2)], 'm--');

plot(bpos(1), bpos(2), 'r+');text(bpos(1)-20, bpos(2), 'b_+','HorizontalAlignment','right','VerticalAlignment','middle');
plot(bneg(1), bneg(2), 'r+');text(bneg(1)-20, bneg(2), 'b_-','HorizontalAlignment','right','VerticalAlignment','middle');

plot(apos(1), apos(2), 'b+');text(apos(1)-20, apos(2), 'a_+','HorizontalAlignment','right','VerticalAlignment','middle');
plot(aneg(1), aneg(2), 'b+');text(aneg(1)-20, aneg(2), 'a_-','HorizontalAlignment','right','VerticalAlignment','middle');

plot(point_center(1), point_center(2), 'g+');text(point_center(1)-20, point_center(2), 'C','HorizontalAlignment','right','VerticalAlignment','middle');
plot(point_vanishing(1), point_vanishing(2), 'g+');text(point_vanishing(1)-20, point_vanishing(2), 'p_v','HorizontalAlignment','right','VerticalAlignment','middle');

plot([pchk(1) point_vanishing(1)], [pchk(2) point_vanishing(2)], 'm--');
plot([ppen(1), point_center(1)], [ppen(2), point_center(2)], 'm--');
axis auto
hold off

% figure(), imshow(imOrig);
end

function [theta_center, theta_vanishing] =  doubleCross(ta,taa,tb,tbb)
% taa, tbb : negative
% ta, tb : positive
theta_center = [
 (2*ta*taa - 2*tb*tbb)/(ta + taa - tb - tbb) + (tb*tbb - ta*taa + (ta^2*taa^2 - ta^2*taa*tb - ta^2*taa*tbb + ta^2*tb*tbb - ta*taa^2*tb - ta*taa^2*tbb + ta*taa*tb^2 + 2*ta*taa*tb*tbb + ta*taa*tbb^2 - ta*tb^2*tbb - ta*tb*tbb^2 + taa^2*tb*tbb - taa*tb^2*tbb - taa*tb*tbb^2 + tb^2*tbb^2)^(1/2))/(ta + taa - tb - tbb)
 (2*ta*taa - 2*tb*tbb)/(ta + taa - tb - tbb) - (ta*taa - tb*tbb + (ta^2*taa^2 - ta^2*taa*tb - ta^2*taa*tbb + ta^2*tb*tbb - ta*taa^2*tb - ta*taa^2*tbb + ta*taa*tb^2 + 2*ta*taa*tb*tbb + ta*taa*tbb^2 - ta*tb^2*tbb - ta*tb*tbb^2 + taa^2*tb*tbb - taa*tb^2*tbb - taa*tb*tbb^2 + tb^2*tbb^2)^(1/2))/(ta + taa - tb - tbb)
];

% center has to greater than > taa and > tbb, < tb, <ta
sel_th1 = theta_center(1) > taa && theta_center(1) > tbb && theta_center(1) < ta  && theta_center(1) < tb
sel_th2 = theta_center(2) > taa && theta_center(2) > tbb && theta_center(2) < ta  && theta_center(2) < tb

assert(xor(sel_th1,sel_th2))

if(sel_th1)
    theta_center = theta_center(1);
else
    theta_center = theta_center(2);
end

theta_vanishing =[
 -(tb*tbb - ta*taa + (ta^2*taa^2 - ta^2*taa*tb - ta^2*taa*tbb + ta^2*tb*tbb - ta*taa^2*tb - ta*taa^2*tbb + ta*taa*tb^2 + 2*ta*taa*tb*tbb + ta*taa*tbb^2 - ta*tb^2*tbb - ta*tb*tbb^2 + taa^2*tb*tbb - taa*tb^2*tbb - taa*tb*tbb^2 + tb^2*tbb^2)^(1/2))/(ta + taa - tb - tbb)
  (ta*taa - tb*tbb + (ta^2*taa^2 - ta^2*taa*tb - ta^2*taa*tbb + ta^2*tb*tbb - ta*taa^2*tb - ta*taa^2*tbb + ta*taa*tb^2 + 2*ta*taa*tb*tbb + ta*taa*tbb^2 - ta*tb^2*tbb - ta*tb*tbb^2 + taa^2*tb*tbb - taa*tb^2*tbb - taa*tb*tbb^2 + tb^2*tbb^2)^(1/2))/(ta + taa - tb - tbb)
];

% vanishing has to be (> ta and > tb) or (< taa and < tbb) 

sel_tv1 = (theta_vanishing(1) > ta && theta_vanishing(1) > tb) || (theta_vanishing(1) < taa && theta_vanishing(1) < tbb);
sel_tv2 = (theta_vanishing(2) > ta && theta_vanishing(2) > tb) || (theta_vanishing(2) < taa && theta_vanishing(2) < tbb);

assert(xor(sel_tv1,sel_tv2))

if(sel_tv1)
    theta_vanishing = theta_vanishing(1);
else
    theta_vanishing = theta_vanishing(2);
end


end

function nx = hnormalise(x)
    
    [rows,npts] = size(x);
    nx = x;

    % Find the indices of the points that are not at infinity
    finiteind = find(abs(x(rows,:)) > eps);

    if length(finiteind) ~= npts
        warning('Some points are at infinity');
    end

    % Normalise points not at infinity
    for r = 1:rows-1
	nx(r,finiteind) = x(r,finiteind)./x(rows,finiteind);
    end
    nx(rows,finiteind) = 1;
end