function crossRatio()
    clear all
    close all

    check_p50h = false; % some additional plot for test
    figurePlot = 6; % 0 to 5, plot different figures for the slides

    destImageSize = 1500;

    i1 = imread('football.jpg');

    %              x         y       w
    % get the four corners (NB: I use ginput to get coordinates)
    p1 = [[0.0300    0.6970] * 1e3, 1];
    p2 = [[0.0810    0.3640] * 1e3, 1];
    p3 = [[1.0710    0.4190] * 1e3, 1];
    p4 = [[0.7960    0.2450] * 1e3, 1];

    p50l = [514   569 1];

    % lines joining points
    l1 = cross(p1,p2);
    l2 = cross(p3,p4);

    % horizontal vanishing
    ph = cross(l1,l2);
    ph(1) = ph(1) / ph(3);
    ph(2) = ph(2) / ph(3);
    ph(3) = 1;

    % lines joining points
    l3 = cross(p1,p3);
    l4 = cross(p2,p4);

    % vertical vanishing
    pv = cross(l3,l4);
    pv(1) = pv(1) / pv(3);
    pv(2) = pv(2) / pv(3);
    pv(3) = 1;

    % compute line at infinity
    l_inf = cross(pv,ph);
    %check that pv and ph are on l_inf
    fprintf('check pv is on l_inf: %f\n',l_inf * pv');
    fprintf('check ph is on l_inf: %f\n',l_inf * ph');


    %% middle vertical lines of the field
    % try this wrong approach, compute medium point as segment medium (see
    % figure in blue)
    pMid1Wrong = (p1(1:2) + p2(1:2)) / 2;
    pMid2Wrong = (p3(1:2) + p4(1:2)) / 2;

    % express line in parametric form
    l1dir = p2(1:2)-p1(1:2);
    thetap21 = norm(l1dir); % theta corresponding to p2
    l1dir = l1dir' / norm(l1dir);
    thetapv1 = norm(ph(1:2)-p1(1:2)); % theta corresponding to vanishing point

    % express line in parametric form
    l2dir = p4(1:2)-p3(1:2);
    thetap22 = norm(l2dir); % theta corresponding to p2
    l2dir = l2dir' / norm(l2dir);
    thetapv2 = norm(ph(1:2)-p3(1:2)); % theta corresponding to vanishing point

    % calculate thetaMid with cross ratio on the two lines
    thetaMid1 = (thetapv1 * thetap21) / (2*thetapv1 - thetap21);
    thetaMid2 = (thetapv2 * thetap22) / (2*thetapv2 - thetap22);

    pMid1 = p1(1:2)' + thetaMid1*l1dir;
    pMid2 = p3(1:2)' + thetaMid2*l2dir;

    %% get the theta on line1 for the field end
    llowdir = p3 - pv;
    theta1  = norm(llowdir);
    llowdir = llowdir ./ norm(llowdir);
    theta2  = norm(p50l - pv);

    theta3 = -theta1*theta2/ (theta2 - 2*theta1);
    pendfieldl = pv + llowdir * theta3;

    % only for check
    if(check_p50h)
        l50h = cross(p50l,ph);
        p50h = cross(l50h,l4);
        p50h = p50h ./ p50h(3);

        lhighdir = p4 - pv;
        theta1h  = norm(lhighdir);
        lhighdir = lhighdir ./ norm(lhighdir);
        theta2h  = norm(p50h - pv);

        theta3h = -theta1h*theta2h/ (theta2h - 2*theta1h);
        pendfieldh = pv + lhighdir * theta3h;

    end

    % compute the end line equation
    lend = cross(pendfieldl,ph);
    fprintf('end line equation: [%f %f %f]\n', lend);
    pendh = cross(l4,lend);
    pendh = pendh ./ pendh(3);

    %% check that vanishing point is on lines joining mid points
    lMid = cross([pMid1;1],[pMid2;1]); 
    fprintf('check mid line pass in pv: %f\n',lMid' * pv');

    lMidWrong = cross([pMid1Wrong';1],[pMid2Wrong';1]); 
    fprintf('check wrong mid line doesn''t pass in pv: %f\n',lMidWrong' * pv');

    %% find the diagonal vanishing point

    ldiag = cross(pendfieldl,p4);
    pvdiag = cross(ldiag,l_inf);
    pvdiag = hnormalise(pvdiag');

    %% show image, lines and text
    set(0,'defaultTextFontName', 'Arial');
    set(0,'defaultTextFontSize', 14);
    set(0,'defaultTextFontWeight', 'Bold');
    set(0,'defaultTextBackgroundColor', [1,1,1]);
    set(0,'defaultLineLineWidth', 3);

    figure(),imshow(i1)
    hold on
    if(figurePlot > 0)
        plot(p1(1),p1(2),'r+');
        plot(p2(1),p2(2),'r+');
        plot(p3(1),p3(2),'r+');
        plot(p4(1),p4(2),'r+');

        if(figurePlot > 1)
            plot([p1(1) ph(1)],[p1(2) ph(2)],'r');
            plot([p3(1) ph(1)],[p3(2) ph(2)],'r');

            plot([p1(1) pv(1)],[p1(2) pv(2)],'r');
            plot([p2(1) pv(1)],[p2(2) pv(2)],'r');
            plot(ph(1),ph(2),'r+');
            plot(pv(1),pv(2),'r+');

            plot([ph(1) pv(1)],[ph(2) pv(2)],'r--');


            if(figurePlot == 3)
                plot(pMid1Wrong(1),pMid1Wrong(2),'b+');
                plot(pMid2Wrong(1),pMid2Wrong(2),'b+');

                dMinWrong = pMid2Wrong - pMid1Wrong;
                theta = 3.75 * norm(dMinWrong);
                dMinWrong = dMinWrong ./ norm(dMinWrong);
                plot([pMid1Wrong(1), pMid1Wrong(1) + theta*dMinWrong(1)],[pMid1Wrong(2), pMid1Wrong(2) + theta*dMinWrong(2)],'b--');
            end

            if(figurePlot == 4)
                plot(pMid1(1),pMid1(2),'g+');

                plot(pMid2(1),pMid2(2),'g+');
                plot([pMid1(1) pv(1)],[pMid1(2) pv(2)],'g');
            end

            if(figurePlot >= 5)
                plot([p1(1) pendfieldl(1)],[p1(2) pendfieldl(2)],'m');

                if(check_p50h)
                    plot([p2(1) pendfieldh(1)],[p2(2) pendfieldh(2)],'m');
                end

                plot([pendfieldl(1) ph(1)],[pendfieldl(2) ph(2)],'m');

                if(check_p50h)
                    plot([p50l(1) p50h(1)],[p50l(2) p50h(2)],'m');
                end



                plot(p50l(1),p50l(2),'m+');
                text(p50l(1),p50l(2)+35,'p_{50}','HorizontalAlignment','right','VerticalAlignment','top');

                if(check_p50h)
                    text(p50h(1),p50h(2),'p_{50H}');
                end

                plot([pendh(1) p2(1)],[pendh(2) p2(2)],'m');


                if(figurePlot == 6)
                    plot(pvdiag(1), pvdiag(2), 'm+');
                    plot([pvdiag(1) pendfieldl(1)], [pvdiag(2)  pendfieldl(2)], 'm--');
                    text(pvdiag(1)-20,pvdiag(2)-10,'p_{d}','HorizontalAlignment','right','VerticalAlignment','bottom');
                end

                text(pendfieldl(1)-10,pendfieldl(2),'p_{end}','HorizontalAlignment','right','VerticalAlignment','middle');
                text(pendh(1)-10,pendh(2),'p^h_{end}','HorizontalAlignment','right','VerticalAlignment','middle');

            end % == 5

            if(figurePlot == 4)
                text(pMid1(1)-20,pMid1(2),'p_{mid1}','HorizontalAlignment','right','VerticalAlignment','middle');
                text(pMid2(1)+10,pMid2(2)-10,'p_{mid2}','HorizontalAlignment','left','VerticalAlignment','bottom');
            end % > 3
            text(ph(1),ph(2)-20,'p_h','HorizontalAlignment','center','VerticalAlignment','bottom');
            text(pv(1)+20,pv(2),'p_v','HorizontalAlignment','left','VerticalAlignment','mid');
            text((ph(1)+pv(1))/2,(ph(2)+pv(2))/2-20,'l_{inf}');

        end % > 1
        text(p1(1)-10,p1(2)+10,'p_1','HorizontalAlignment','right','VerticalAlignment','top');
        text(p2(1)-10,p2(2)-10,'p_2','HorizontalAlignment','right','VerticalAlignment','bottom');
        text(p3(1)+10,p3(2)+10,'p_3','HorizontalAlignment','left','VerticalAlignment','top');
        text(p4(1)+10,p4(2)-10,'p_4','HorizontalAlignment','left','VerticalAlignment','bottom');
    end
    axis auto
    hold off

    %% player position
    player = [742,404,1]; % an example
    % player = [440.3135,  416.7605, 1];% check with center position
    % player = p3; % test point
    % player = p4; % test point
    % player = pendfieldl; % test point
    % player = pendh; % test point

    % find lines from vanishing points to player
    lph = cross(player,ph); 
    lpv = cross(player,pv); 


    % find the intersection with 
    playery = cross(lph, l3); % long side
    playery = hnormalise(playery');
    playerx = cross(lpv, l2); % short side
    playerx = hnormalise(playerx');

    thetax = norm(playerx - p3');
    thetae = norm(p4 - p3);
    thetam = norm([pMid2' 1] - p3);
    thetaPlayerX = thetax*(thetae-thetam)/(thetam*(thetae-thetax) + thetax*(thetae - thetam));

    thetax = norm(playery - p3');
    thetae = norm(pendfieldl - p3);
    thetam = norm(p50l - p3);
    thetaPlayerY = thetax*(thetae-thetam)/(thetam*(thetae-thetax) + thetax*(thetae - thetam));

    fprintf('player relative position : %f,%f\n',thetaPlayerX,thetaPlayerY);

    figure(),imshow(i1)
    hold on

    circle(player(1:2),4)
    text(player(1)-20,player(2),'P','HorizontalAlignment','right','VerticalAlignment','middle');

    plot([playery(1) ph(1)],[playery(2) ph(2)],'m');
    plot([playerx(1) pv(1)],[playerx(2) pv(2)],'m');
    plot([player(1) playerx(1)],[player(2) playerx(2)],'m--');

    plot([p3(1) pendfieldl(1)],[p3(2) pendfieldl(2)],'r');
    plot([p4(1) pendh(1)],[p4(2) pendh(2)],'r');

    plot([pendfieldl(1) pendh(1)],[pendfieldl(2) pendh(2)],'r');
    plot([p3(1) p4(1)],[p3(2) p4(2)],'r');

    plot(ph(1),ph(2),'r+');
    plot(pv(1),pv(2),'r+');

    plot(p50l(1),p50l(2),'m+');
    plot(pMid2(1),pMid2(2),'m+');

    text(pMid2(1)+10,pMid2(2)-10,'p_{mid2}','HorizontalAlignment','left','VerticalAlignment','bottom');

    text(p50l(1),p50l(2)+35,'p_{50}','HorizontalAlignment','right','VerticalAlignment','top');

    text(pendfieldl(1)-10,pendfieldl(2),'p_{end}','HorizontalAlignment','right','VerticalAlignment','middle');

    text(ph(1),ph(2)-20,'p_h','HorizontalAlignment','center','VerticalAlignment','bottom');
    text(pv(1)+20,pv(2),'p_v','HorizontalAlignment','left','VerticalAlignment','mid');

    text(p1(1)-10,p1(2)+10,'p_1','HorizontalAlignment','right','VerticalAlignment','top');
    text(p2(1)-10,p2(2)-10,'p_2','HorizontalAlignment','right','VerticalAlignment','bottom');
    text(p3(1)+10,p3(2)+10,'p_3','HorizontalAlignment','left','VerticalAlignment','top');
    text(p4(1)+10,p4(2)-10,'p_4','HorizontalAlignment','left','VerticalAlignment','bottom');
    axis auto
    hold off


    %% affine rectification
    l_inf = l_inf ./ l_inf(3);
    H = eye(3);
    H(3,:) = l_inf;
    l_inf_real = inv(H)' * l_inf';
    fprintf('check the new line at infinity is the real one: [%f %f %f]\n', l_inf_real);

    region = [ 
        max(min([p1(2),p2(2),p3(2),p4(2)])-100,1) 
        min(max([p1(2),p2(2),p3(2),p4(2)])+100,size(i1,1)) 
        max(min([p1(1),p2(1),p3(1),p4(1)])-100,1) 
        min(max([p1(1),p2(1),p3(1),p4(1)])+100,size(i1,2))
    ];


    [i2, newT] = imTrans(i1,H,region,destImageSize);
    % figure(1)
    % hold on
    % plot([region(3) region(4) region(4) region(3) region(3)],[region(2) region(2) region(1) region(1) region(2)],'b');
    % 
    % region2 = inv(newT) * H * [region(3) region(4) region(4) region(3) region(3);region(2) region(2) region(1) region(1) region(2); 1 1 1 1 1];
    % region2 = hnormalise(region2);

    % transform points
    pointsNew = inv(newT) * H * [p1;p2;p4;p3;p1]';
    pointsNew = hnormalise(pointsNew);

    i2 = imcrop(i2,[min(pointsNew(1,:)),min(pointsNew(2,:)),max(pointsNew(1,:)),max(pointsNew(2,:))]);

    pointsNew = inv([1 0 min(pointsNew(1,:))
                 0 1 min(pointsNew(2,:))
                 0 0 1]) * pointsNew;

    figure(),imshow(i2)
    hold on
    % plot(region2(1,:),region2(2,:),'b');
    plot(pointsNew(1,:),pointsNew(2,:),'r');
    axis auto
    hold off

    %% check parallelism
    l1n = inv(H)' * l1';
    l2n = inv(H)' * l2';
    phn = cross(l1n,l2n);
    l3n = inv(H)' * l3';
    l4n = inv(H)' * l4';
    pvn = cross(l3n,l4n);

    fprintf('phn = l1n x l2n = %f %f %f\n', phn);
    fprintf('pvn = l3n x l4n = %f %f %f\n', pvn);
end

function nx = hnormalise(x)
    
    [rows,npts] = size(x);
    nx = x;

    % Find the indices of the points that are not at infinity
    finiteind = find(abs(x(rows,:)) > eps);

    if length(finiteind) ~= npts
        warning('Some points are at infinity');
    end

    % Normalise points not at infinity
    for r = 1:rows-1
	nx(r,finiteind) = x(r,finiteind)./x(rows,finiteind);
    end
    nx(rows,finiteind) = 1;
end

function circle(c, r, nsides, col)
    
    if nargin == 2
	nsides = 16;
    end
    
    if nargin < 4
	col = [0 0 1];
    end
        
    nsides = max(round(nsides),3); % Make sure it is an integer >= 3
    
    a = [0:2*pi/nsides:2*pi];
    line(r*cos(a)+c(1), r*sin(a)+c(2), 'color', col);
end