function vanishing3D()
    clear all
    close all

    set(0,'defaultTextFontName', 'Arial');
    set(0,'defaultTextFontSize', 14);
    set(0,'defaultTextFontWeight', 'Bold');
    set(0,'defaultTextBackgroundColor', [1,1,1]);
    set(0,'defaultLineLineWidth', 3);


    check_p50h = false; % some additional plot for test
    figurePlot = 4; % 0 to 5, plot different figures for the slides

    destImageSize = 1500;

    i1 = imread('3pt.jpg');

% NB: use nan as separator for line plotting
    pz1 = [158   158  nan 247 223 nan 262 239  nan  20  41   % x  
           288   128  nan 334 167 nan 304 172  nan 354 238   % y
             1     1  nan 1   1   nan   1   1  nan   1   1];    % w

    py1 = [142    10  nan 145 40  nan 140   24     % x  
           317   394  nan  86 218 nan 401  441   % y
             1     1  nan 1   1   nan   1    1];    % w

    px1 = [ 145.69   244.23  nan 146.29  239.39  nan  192.84  226.09 nan  159.60  174.10
             86.04   150.72  nan 112.03  168.86  nan   82.41  105.38 nan  396.16  399.19
             1       1       nan      1    1     nan    1       1    nan     1      1];


     % vertical vanishing
    pvz = lineIntersectLeastSquare(linesFromPointCouple(pz1));
     % y vanishing point
    pvy = lineIntersectLeastSquare(linesFromPointCouple(py1));
     % x vanishing point
    pvx = lineIntersectLeastSquare(linesFromPointCouple(px1));

    pvz_pz1 = pz1;
    pvz_pz1(:,1:3:end) = repmat(pvz,1, ceil(size(pz1,2)/3));

    pvy_py1 = py1;
    pvy_py1(:,1:3:end) = repmat(pvy,1, ceil(size(py1,2)/3));

    pvx_px1 = px1;
    pvx_px1(:,1:3:end) = repmat(pvx,1, ceil(size(px1,2)/3));
    
    figure()
    imshow(i1)
    hold on
    plot(pz1(1,:),pz1(2,:) ,'r');

    plot(py1(1,:),py1(2,:) ,'b');
    
    plot(px1(1,:),px1(2,:) ,'g');
    
    plot(pvz_pz1(1,:),pvz_pz1(2,:) ,'m--');
    plot(pvy_py1(1,:),pvy_py1(2,:) ,'m--');
    plot(pvx_px1(1,:),pvx_px1(2,:) ,'m--');

    plot(pvz(1),pvz(2,:) ,'r+');text(pvz(1),pvz(2)-30,'V_{z}','HorizontalAlignment','center','VerticalAlignment','bottom');
    plot(pvy(1),pvy(2,:) ,'b+');text(pvy(1)-30,pvy(2),'V_{y}','HorizontalAlignment','right','VerticalAlignment','middle');
    plot(pvx(1,:),pvx(2,:) ,'g+');text(pvx(1)+30,pvx(2),'V_{x}','HorizontalAlignment','left','VerticalAlignment','middle');

    plot([pvx(1) pvy(1)],[pvx(2,:) pvy(2,:)] ,'k');text(0.5*(pvx(1)+pvy(1)),0.5*(pvx(2)+pvy(2)),'l_{xy}','HorizontalAlignment','center','VerticalAlignment','middle');
    plot([pvx(1) pvz(1)],[pvx(2,:) pvz(2,:)] ,'k');text(0.5*(pvx(1)+pvz(1)),0.5*(pvx(2)+pvz(2)),'l_{xz}','HorizontalAlignment','center','VerticalAlignment','middle');
    plot([pvy(1) pvz(1)],[pvy(2,:) pvz(2,:)] ,'k');text(0.5*(pvy(1)+pvz(1)),0.5*(pvy(2)+pvz(2)),'l_{yz}','HorizontalAlignment','center','VerticalAlignment','middle');
    
    axis auto
    hold off
end

function ls = linesFromPointCouple(ps)
    n = ceil(size(ps,2)/3);
    ls = nan(3,n);
    for i = 1 : n
        p1 = ps(:,(i-1) * 3 + 1);
        p2 = ps(:,(i-1) * 3 + 2);
        ls(:,i) = calcLinesJoining(p1,p2);
    end
end

function l = calcLinesJoining(p1,p2)
    l = cross(p1,p2);
end

function p = lineIntersectLeastSquare(lines)
% solve the system ai x + bi y + ci = 0 in (x,y)
% where ai,bi,ci are the coefficient of i-th line
% then 
% 
% A = |a1 b1|
%     |a2 b2| 
%     |.....| 
%     |an bn| 
% 
% x = [x y]'; 
% b = |-c1|
%     |-c2|
%     |...|
%     |-cn|;
% x = A\b; find the least square solution

% NB: this is not a "good" solution, but is ok with our purposes see 8.6 of "Multiple View Geometry in computer vision" for
% more details.

    % lines : 3 x n
    A = nan(size(lines,2),2);
    b = nan(size(lines,2),1);
    for i = 1 : size(lines,2)
        A(i,1:2) = lines(1:2,i)';
        b(i) = -lines(3,i)';
    end
    p = A\b;
    p = [p;1];
end